package cn.com.qmth.examcloud.task.service.impl;

import java.lang.reflect.Method;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.api.HandleSyncCloudService;
import cn.com.qmth.examcloud.commons.api.request.SyncBaseRequest;
import cn.com.qmth.examcloud.commons.api.request.SyncCourseReq;
import cn.com.qmth.examcloud.commons.api.request.SyncExamReq;
import cn.com.qmth.examcloud.commons.api.request.SyncExamStudentReq;
import cn.com.qmth.examcloud.commons.api.request.SyncOrgReq;
import cn.com.qmth.examcloud.commons.api.request.SyncSpecialtyReq;
import cn.com.qmth.examcloud.commons.api.request.SyncStudentReq;
import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.JsonUtil;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.base.util.RegExpUtil;
import cn.com.qmth.examcloud.task.dao.DataSyncRepo;
import cn.com.qmth.examcloud.task.dao.entity.DataSyncEntity;
import cn.com.qmth.examcloud.task.service.DataSyncService;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年9月19日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class DataSyncServiceImpl implements DataSyncService {

	@Autowired
	DataSyncRepo dataSyncRepo;

	@Autowired
	HandleSyncCloudService handleSyncCloudService;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param methodName
	 * @param req
	 */
	private void sync(String methodName, SyncBaseRequest req) {
		String group = PropertiesUtil.getString("$sync." + methodName + ".group");
		if (StringUtils.isBlank(group)) {
			throw new StatusException("T-001001", "group is not configured");
		}

		List<String> componentList = RegExpUtil.findAll(group, "[^\\,]+");

		if (CollectionUtils.isEmpty(componentList)) {
			throw new StatusException("T-001001", "no component");
		}

		for (String component : componentList) {
			String url = PropertiesUtil
					.getString("$sync." + methodName + ".component." + component);
			req.setUrl(url);
			try {
				Method method = handleSyncCloudService.getClass().getMethod(methodName,
						req.getClass());
				method.invoke(handleSyncCloudService, req);
			} catch (Exception e) {
				DataSyncEntity entity = new DataSyncEntity();
				entity.setComponent(component);
				entity.setMethodName(methodName);
				entity.setParamType(req.getClass().getName());
				entity.setSyncNum(0);
				entity.setParamJson(JsonUtil.toJson(req));
				dataSyncRepo.saveAndFlush(entity);
			}
		}
	}

	@Override
	public void syncCourse(SyncCourseReq req) {
		sync("syncCourse", req);
	}

	@Override
	public void syncOrg(SyncOrgReq req) {
		sync("syncOrg", req);
	}

	@Override
	public void syncStudent(SyncStudentReq req) {
		sync("syncStudent", req);
	}

	@Override
	public void syncExamStudent(SyncExamStudentReq req) {
		sync("syncExamStudent", req);
	}

	@Override
	public void syncSpecialty(SyncSpecialtyReq req) {
		sync("syncSpecialty", req);
	}

	@Override
	public void syncExam(SyncExamReq req) {
		sync("syncExam", req);
	}

}
